package	CheckReferer;

=head1 NAME

CheckReferer.pm - Check referer or remote host.

$Id: CheckReferer.pm,v 1.2 2003/01/03 12:04:10 t_shigetome Exp $

=head1 SYNOPSIS

=head1 DESCRIPTION

This module use for check CGI referer.

=head2 METHOD

=over 4

=item CheckReferer( $filename )

Verify referer with permitted URIs.
The $filename is filename of parameter file.
That parameter file will be parsed with F<Params.pm>.

=item CheckForbiddenRemote( $filename )

Check forbidden remote hosts.
The $filename is filename of parameter file.
That parameter file will be parsed with F<Params.pm>.

=item CheckAcceptRemote( $filename )

Verify with accepted remote hosts.
The $filename is filename of parameter file.
That parameter file will be parsed with F<Params.pm>.

=back

=head2 Parameter file

Parameter file format is following.
That `label' is meaningless.

	referer label{
		accepted referer URIs;
		.
		.
	};
	forbidden label{
		forbidden remote host;
		.
		.
	};
	accept label{
		accepted remote host;
		.
		.
	};

=head1 SEE ALSO

F<Params.pm>, F<Timestr.pm>

=head1 AUTHOR

Tetsuya Shigetome, E<lt>t_shigetome@muf.biglobe.ne.jpE<gt>.

=head1 HISTORY

=over 4

=item $Revision: 1.2 $

$Log: CheckReferer.pm,v $
Revision 1.2  2003/01/03 12:04:10  t_shigetome

o No need timestr.pl

o Correct typo


=item Revision: 1.1 

Log: CheckReferer.pm,v
Revision 1.1  2002/11/25 11:49:25  t_shigetome

o First release


=back

=cut

use strict;
use Carp;
use Params;
use Timestr;
require Exporter;
@CheckReferer::ISA = qw(Exporter);

# symbols to export by default
@CheckReferer::EXPORT = qw( CheckReferer CheckForbiddenRemote CheckAcceptRemote);
# symbols to export on request
@Params::EXPORT_OK = qw(&CheckReferer &CheckForbiddenRemote &CheckAcceptRemote $CheckReferer::referer $CheckReferer::address $CheckReferer::host );

$CheckReferer::VERSION = sprintf("%d.%02d", q$Revision: 1.2 $ =~ /(\d+)\.(\d+)/);

$CheckReferer::referer = $ENV{'HTTP_REFERER'};
$CheckReferer::address = $ENV{'REMOTE_ADDR'};
$CheckReferer::host = $ENV{'REMOTE_HOST'};
{
	$CheckReferer::host	= $ENV{'REMOTE_ADDR'} unless( defined($CheckReferer::host) );
	if( $CheckReferer::host =~ m/[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}/ )
	{
		$CheckReferer::host = (gethostbyaddr(pack("C4", split(/\./, $CheckReferer::host)), 2))[0];
	}
	$CheckReferer::host = $ENV{'REMOTE_ADDR'} unless( defined($CheckReferer::host) && $CheckReferer::host ne "" );
}
##############################
#
sub CheckReferer
{
my($file) = @_;
my(@caller) = caller;
my($para, $rh_home);

	croak "Do not exist: $file" unless( -e $file );

	$para = new Params($file);
	&CheckForbidden($para);
	# Check of Referer
	$rh_home = $para->GetData('referer');
	foreach( @{$para->ailias} )
	{
		$rh_home->{$_} =~ s/$Params::delimiter/|/g;
		return $_ if $CheckReferer::referer =~ m/($rh_home->{$_})/;
	}

	carp "Unknown referer $CheckReferer::referer remote address $CheckReferer::address host $CheckReferer::host";
	return 0;
}
##############################
#
sub CheckForbiddenRemote
{
my($file) = @_;
my($para);
	croak "Do not exist: $file" unless( -e $file );
	$para = new Params($file);
	return &CheckForbidden($para);
}
sub CheckForbidden
{
my($para) = @_;
my($rh_deny);
	# Check for Forbidden 
	$rh_deny = $para->GetData('forbidden');
	foreach( @{$para->ailias()} )
	{
		$rh_deny->{$_} =~ s/$Params::delimiter/|/g;
		if( $CheckReferer::host =~ m/($rh_deny->{$_})/ )
		{
my($now) = &Timestr::GetTimeStr($^T);
			print <<"_Forbidden_Access_";
Status: 403 Forbidden
Content-type: text/plain

YOUR ACCESS FORBIDDEN

_Forbidden_Access_
			carp "Access Forbidden $CheckReferer::host($CheckReferer::address) to $0 at time $now";
			return	0;
		}
	}
	return 1;
}
#################
sub CheckAcceptRemote
{
my($file) = @_;
my($para);
	croak "Do not exist: $file" unless( -e $file );
	$para = new Params($file);
	return &CheckAccept($para);
}
sub CheckAccept
{
my($para) = @_;
my($rh_accept);
	# Check for Accept
	$rh_accept = $para->GetData('accept');
	foreach( @{$para->ailias()} )
	{
		$rh_accept->{$_} =~ s/$Params::delimiter/|/g;
		return $_ if( $CheckReferer::host =~ m/($rh_accept->{$_})/ )
	}
my($now) = &Timestr::GetTimeStr($^T);
	print <<"_Forbidden_Access_";
Status: 403 Forbidden
Content-type: text/plain

YOUR ACCESS FORBIDDEN

_Forbidden_Access_
	carp "Access do not accept $CheckReferer::host($CheckReferer::address) to $0 at time $now\n";
	return	0;
}
1; #return true 
